@@ -7,6 +7,8 @@ import sys, os
 import locale
 import unittest
 
+import lg_testutils # Found in the same directory of this test script
+
 # Show information on this program run
 print('Running by:', sys.executable)
 print('Running {} in:'.format(sys.argv[0]), os.getcwd())
@@ -20,7 +22,7 @@ from linkgrammar import Sentence, Linkage, ParseOptions, Link, Dictionary, \
 
 
 # Show the location and version of the bindings modules
-for module in 'linkgrammar', '_clinkgrammar':
+for module in 'linkgrammar', '_clinkgrammar', 'lg_testutils':
     if module in sys.modules:
         print("Using", sys.modules[module], end='')
         if hasattr(sys.modules[module], '__version__'):
@@ -35,11 +37,9 @@ def setUpModule():
     if datadir:
         clg.dictionary_set_data_dir(datadir)
 
-    clg.test_data_srcdir = os.getenv("srcdir")
+    clg.test_data_srcdir = os.getenv("srcdir", os.path.dirname(sys.argv[0]))
     if clg.test_data_srcdir:
         clg.test_data_srcdir += "/"
-    else:
-        clg.test_data_srcdir = ""
 
 # The tests are run in alphabetical order....
 #
@@ -211,18 +211,31 @@ class CParseOptionsTestCase(unittest.TestCase):
 
 class DBasicParsingTestCase(unittest.TestCase):
     def setUp(self):
-        self.d, self.po = Dictionary(), ParseOptions()
+        self.d = Dictionary()
+        self.po = None
 
-    def parse_sent(self, text):
-        return list(Sentence(text, self.d, self.po).parse())
+    def parse_sent(self, text, po=ParseOptions()):
+        return list(Sentence(text, self.d, po).parse())
 
     def test_that_parse_returns_empty_iterator_on_no_linkage(self):
+        """Parsing a bad sentence with no null-links shouldn't give any linkage."""
         result = self.parse_sent("This this doesn't parse")
         linkage_exists = False
         for _ in result:
             linkage_exists = True
             self.assertFalse(linkage_exists, "Unparsable sentence has linkages.")
 
+    def test_that_parse_returns_empty_iterator_on_no_linkage_sat(self):
+        """Parsing a bad sentence with no null-links shouldn't give any linkage (sat)"""
+        self.po = ParseOptions(use_sat=True)
+        if self.po.use_sat != True:
+            raise unittest.SkipTest("Library not configured with SAT parser")
+        result = self.parse_sent("This this doesn't parse", self.po)
+        linkage_exists = False
+        for _ in result:
+            linkage_exists = True
+            self.assertFalse(linkage_exists, "SAT: Unparsable sentence has linkages.")
+
     def test_that_parse_sent_returns_list_of_linkage_objects_for_valid_sentence(self):
         result = self.parse_sent("This is a relatively simple sentence.")
         self.assertTrue(isinstance(result[0], Linkage))
@@ -274,6 +287,12 @@ class DBasicParsingTestCase(unittest.TestCase):
              ['LEFT-WALL', 'is.v', 'it', 'fine.a', '?', 'RIGHT-WALL'])
         locale.setlocale(locale.LC_CTYPE, oldlocale)
 
+    # If \w is supported, other \ shortcuts are hopefully supported too.
+    def test_regex_class_shortcut_support(self):
+        r"""Test that regexes support \w"""
+        linkage = self.parse_sent("This is a _regex_ive regex test")[0]
+        self.assertEqual(linkage.word(4), '_regex_ive[!].a')
+
 class ESATsolverTestCase(unittest.TestCase):
     def setUp(self):
         self.d, self.po = Dictionary(lang='en'), ParseOptions()
@@ -311,7 +330,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
         self.assertEqual(linkage.link(3),
                          Link(linkage, 3, 'this.p','Ss*b','Ss','is.v'))
         self.assertEqual(linkage.link(4),
-                         Link(linkage, 4, 'is.v','O*t','Os','sentence.n'))
+                         Link(linkage, 4, 'is.v','O*m','Os','sentence.n'))
         self.assertEqual(linkage.link(5),
                          Link(linkage, 5, 'a','Ds**c','Ds**c','sentence.n'))
         self.assertEqual(linkage.link(6),
@@ -364,7 +383,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
               '\'s.p', 'shoe.n', 'fell.v-d', 'off', '.', 'RIGHT-WALL'])
 
         self.assertEqual(list(self.parse_sent('Jumbo sat down.')[0].words()),
-             ['LEFT-WALL', 'Jumbo[!]', 'sat.v-d', 'down.a', '.', 'RIGHT-WALL'])
+             ['LEFT-WALL', 'Jumbo[!]', 'sat.v-d', 'down.r', '.', 'RIGHT-WALL'])
 
         # Red is in dict, lower-case, as noun, too.
         # There's no way to really know, syntactically, that Red
@@ -384,7 +403,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
              ['LEFT-WALL', 'May.f', '\'s.v', 'going.v', '?', 'RIGHT-WALL'])
 
         self.assertEqual(list(self.parse_sent('May sat down.')[0].words()),
-             ['LEFT-WALL', 'May.f', 'sat.v-d', 'down.e', '.', 'RIGHT-WALL'])
+             ['LEFT-WALL', 'May.f', 'sat.v-d', 'down.r', '.', 'RIGHT-WALL'])
 
         # McGyver is not in the dict, but is regex-matched.
         self.assertEqual(list(self.parse_sent('McGyver\'s going?')[0].words()),
@@ -395,7 +414,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
               '\'s.p', 'shoe.n', 'fell.v-d', 'off', '.', 'RIGHT-WALL'])
 
         self.assertEqual(list(self.parse_sent('McGyver sat down.')[0].words()),
-             ['LEFT-WALL', 'McGyver[!]', 'sat.v-d', 'down.a', '.', 'RIGHT-WALL'])
+             ['LEFT-WALL', 'McGyver[!]', 'sat.v-d', 'down.r', '.', 'RIGHT-WALL'])
 
         self.assertEqual(list(self.parse_sent('McGyver Industries stock declined.')[0].words()),
              ['LEFT-WALL', 'McGyver[!]', 'Industries[!]',
@@ -471,6 +490,30 @@ class ZENLangTestCase(unittest.TestCase):
     def test_quotes(self):
         linkage_testfile(self, self.d, self.po, 'quotes')
 
+    def test_null_link_range_starting_with_zero(self):
+        """Test parsing with a minimal number of null-links, including 0."""
+        # This sentence has no complete linkage. Validate that the library
+        # doesn't mangle parsing with null-count>0 due to power_prune()'s
+        # connector-discard optimization at null-count==0.  Without commit
+        # "Allow calling chart_parse() with and w/o nulls", the number of
+        # linkages here is 1 instead of 2 and the unused_word_cost is 5.
+        self.po = ParseOptions(min_null_count=0, max_null_count=999)
+        linkages = Sentence('about people attended', self.d, self.po).parse()
+        self.assertEqual(len(linkages), 2)
+        self.assertEqual(linkages.next().unused_word_cost(), 1)
+        # Expected parses:
+        # 1:
+        #    +------------>WV------------>+
+        #    +--------Wd-------+----Sp----+
+        #    |                 |          |
+        #LEFT-WALL [about] people.p attended.v-d
+        # 2:
+        #
+        #            +----Sp----+
+        #            |          |
+        #[about] people.p attended.v-d
+
+
 class ZENConstituentsCase(unittest.TestCase):
     def setUp(self):
         self.d, self.po = Dictionary(lang='en'), ParseOptions()
@@ -620,4 +663,7 @@ def warning(*msg):
     print("{}: Warning:".format(progname), *msg, file=sys.stderr)
 
 
+# Decorate Sentence.parse with eqcost_soretd_parse.
+lg_testutils.add_eqcost_linkage_order(Sentence)
+
 unittest.main()
