@@ -46,7 +46,7 @@ class ParseOptions(object):
         self.disjunct_cost = disjunct_cost
 
     def __del__(self):
-        if hasattr(self, '_obj') and clg:
+        if hasattr(self, '_obj'):
             clg.parse_options_delete(self._obj)
             del self._obj
 
@@ -253,15 +253,31 @@ class ParseOptions(object):
         clg.parse_options_set_all_short_connectors(self._obj, 1 if value else 0)
 
 
+class LG_DictionaryError(Exception):
+    pass
+
 class Dictionary(object):
     def __init__(self, lang='en'):
         self._obj = clg.dictionary_create_lang(lang)
+        if not self._obj:
+            raise LG_DictionaryError('Error: Failed to open dictionary {!r}'.format(lang))
+
+    def __str__(self):
+        return clg.dictionary_get_lang(self._obj)
 
     def __del__(self):
-        if hasattr(self, '_obj') and clg:
+        if hasattr(self, '_obj'):
             clg.dictionary_delete(self._obj)
             del self._obj
 
+    def __nonzero__(self):
+        """Return False iff the dictionary could not be opened or has been closed"""
+        if not hasattr(self, '_obj'):
+            return False
+        return bool(self._obj)
+
+    __bool__ = __nonzero__      # Account python3
+
     def linkgrammar_get_dict_version(self):
         return clg.linkgrammar_get_dict_version(self._obj)
 
@@ -307,7 +323,7 @@ class Linkage(object):
         self._obj = clg.linkage_create(idx, sentence._obj, parse_options)
 
     def __del__(self):
-        if hasattr(self, '_obj') and clg:
+        if hasattr(self, '_obj'):
             clg.linkage_delete(self._obj)
             del self._obj
 
@@ -385,7 +401,7 @@ class Sentence(object):
         self._obj = clg.sentence_create(self.text, self.dict._obj)
 
     def __del__(self):
-        if hasattr(self, '_obj') and clg:
+        if hasattr(self, '_obj'):
             clg.sentence_delete(self._obj)
             del self._obj
 
@@ -429,6 +445,9 @@ class Sentence(object):
                 return iter(())
             return self
 
+        def __len__(self):
+            return clg.sentence_num_linkages_found(self.sent._obj)
+
         def next(self):
             if self.num == clg.sentence_num_valid_linkages(self.sent._obj):
                 raise StopIteration()
