@@ -13,12 +13,15 @@
 
 #include <ctype.h>
 #include <limits.h>
-#include <locale.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdarg.h>
+#include <locale.h>
+#ifdef HAVE_LOCALE_T_IN_XLOCALE_H
+#include <xlocale.h>
+#endif /* HAVE_LOCALE_T_IN_XLOCALE_H */
 
 #ifndef _WIN32
 	#include <unistd.h>
@@ -826,11 +829,11 @@ locale_t newlocale_LC_CTYPE(const char *locale)
 {
 	locale_t locobj;
 #ifdef _WIN32
-		locobj = _create_locale(LC_CTYPE, locale);
+	locobj = _create_locale(LC_CTYPE, locale);
 #else
-		locobj = newlocale(LC_CTYPE_MASK, locale, (locale_t)0);
+	locobj = newlocale(LC_CTYPE_MASK, locale, (locale_t)0);
 #endif /* _WIN32 */
-		return locobj;
+	return locobj;
 }
 #endif /* HAVE_LOCALE_T */
 
@@ -838,7 +841,7 @@ locale_t newlocale_LC_CTYPE(const char *locale)
  * Check that the given locale known by the system.
  * In case we don't have locale_t, actually set the locale
  * in order to find out if it is fine. This side effect doesn't cause
- * harm, as the local would be set up to that value anyway shortly.
+ * harm, as the locale would be set up to that value anyway shortly.
  * @param locale Locale string
  * @return True if known, false if unknown.
  */
@@ -849,7 +852,7 @@ bool try_locale(const char *locale)
 		if ((locale_t)0 == ltmp) return false;
 		freelocale(ltmp);
 #else
-		lgdebug(D_USER_FILES, "Debug: Setting program's locale %s", locale);
+		lgdebug(D_USER_FILES, "Debug: Setting program's locale \"%s\"", locale);
 		if (NULL == setlocale(LC_CTYPE, locale))
 		{
 			lgdebug(D_USER_FILES, " failed!\n");
@@ -867,7 +870,7 @@ bool try_locale(const char *locale)
 void set_utf8_program_locale(void)
 {
 #ifndef _WIN32
-	/* The LG library doesn't use mbrtowc_l(), since it doesn't exists in
+	/* The LG library doesn't use mbrtowc_l(), since it doesn't exist in
 	 * the dynamic glibc (2.22). mbsrtowcs_l() could also be used, but for
 	 * some reason it exists only in the static glibc.
 	 * In order that mbrtowc() will work for any UTF-8 character, UTF-8
@@ -879,7 +882,7 @@ void set_utf8_program_locale(void)
 		/* Avoid an initial spurious message. */
 		if ((0 != strcmp(locale, "C")) && (0 != strcmp(locale, "POSIX")))
 		{
-			prt_error("Warning: Program locale %s (codeset %s) was not UTF-8; "
+			prt_error("Warning: Program locale \"%s\" (codeset %s) was not UTF-8; "
 						 "force-setting to en_US.UTF-8", locale, codeset);
 		}
 		locale = setlocale(LC_CTYPE, "en_US.UTF-8");
@@ -943,9 +946,9 @@ char * get_default_locale(void)
 	if (NULL != *evname)
 	{
 		locale = ev;
-		lgdebug(D_USER_FILES, "Debug: Environment locale %s=%s\n", *evname, ev);
+		lgdebug(D_USER_FILES, "Debug: Environment locale \"%s=%s\"\n", *evname, ev);
 #ifdef _WIN32
-		/* If compiled with MSVC/MSYS, we still support running under Cygwin. */
+		/* If compiled with MSVC/MinGW, we still support running under Cygwin. */
 		const char *ostype = getenv("OSTYPE");
 		if ((NULL != ostype) && (0 == strcmp(ostype, "cygwin")))
 		{
@@ -964,7 +967,7 @@ char * get_default_locale(void)
 		if (NULL == locale)
 			lgdebug(D_USER_FILES, "Debug: Cannot find user default locale\n");
 		else
-			lgdebug(D_USER_FILES, "Debug: User default locale %s\n", locale);
+			lgdebug(D_USER_FILES, "Debug: User default locale \"%s\"\n", locale);
 		return locale; /* Already strdup'ed */
 #endif /* _WIN32 */
 	}
