@@ -13,12 +13,9 @@
 #ifndef _LINK_GRAMMAR_UTILITIES_H_
 #define _LINK_GRAMMAR_UTILITIES_H_
 
-/* The _Win32 definitions are for native-Windows compilers.
- * This includes MSVC (only version >=14 is supported) and MINGW (under
- * MSYS or Cygwin). The _WIN32 definitions are not for Cygwin, which
- * doesn't define _WIN32.
- * FIXME: MinGW still has problems with UTF-8 output.
- */
+/* The _Win32 definitions are for native-Windows compilers.  This includes
+ * MSVC (only versions >=14 are supported) and MINGW (under MSYS or Cygwin).
+ * The _WIN32 definitions are not for Cygwin, which doesn't define _WIN32. */
 
 #include <ctype.h>
 #include <stdio.h>
@@ -29,6 +26,10 @@
 #include <string.h>
 #include <wchar.h>
 #include <wctype.h>
+#include <locale.h>
+#ifdef HAVE_LOCALE_T_IN_XLOCALE_H
+#include <xlocale.h>
+#endif /* HAVE_LOCALE_T_IN_XLOCALE_H */
 
 #include "error.h"
 
@@ -77,6 +78,10 @@ void *alloca (size_t);
 #define qsort(x, y, z, w) qsort((void *)x, y, z, w)
 #endif /* _MSC_VER */
 
+#if defined(HAVE_LOCALE_T_IN_LOCALE_H) || defined(HAVE_LOCALE_T_IN_XLOCALE_H)
+#define HAVE_LOCALE_T 1
+#endif /* HAVE_LOCALE_T_IN_LOCALE_H || HAVE_LOCALE_T_IN_XLOCALE_H) */
+
 #ifdef _WIN32
 #include <windows.h>
 #include <mbctype.h>
